% Codes for the paper "Quantifying the Risk of Deflation" by Kilian and Manganelli
% by Simone Manganelli
% European Central Bank
% Frankfurt
% Created: 30 October 2002
% Last modified: 18 November 2005

clear, clc
% Load data
% Choose data from 1960 to end of 2001 and generate yearly inflation rates
US = load('j:\papers\inflation\data\cpi_US_SA.txt');
GE = load('j:\papers\inflation\data\cpiDE_OECD_SA_2002.txt');
JP = load('j:\papers\inflation\data\cpiJP_OECD_SA.txt');
OIL = load('j:\papers\inflation\data\oilpriceWTI.txt'); % Source FRED Saint Louis
M3US=load('j:\papers\inflation\data\m1_US_SA.txt');
M2=load('j:\papers\inflation\data\M3_DEBUBA_M2_JP_SA_OECD.txt'); TIMEM=M2(:,1); MGE=M2(:,2); MJP=M2(:,3);
FX = load('j:\papers\inflation\data\fx_dm_yen.txt'); TIMEFX=FX(:,1); DM=FX(:,2); YEN=FX(:,3);

indexOIL = find(OIL(:,1)>1960); OIL = OIL(indexOIL(1:end),:);
timeOIL = OIL(:,1); cpiOILUS = OIL(:,2);
cpiOILDM = cpiOILUS./DM; cpiOILYEN = cpiOILUS./YEN;

indexMUS = find(M3US(:,1)>1960); MUS=M3US(indexMUS(1:end-1),2);

% Throw away October 2002 observation for US and GE and pre 1960 observations for US
% Our sample size is 1960.1 to 2002.9 monthly data
indexUS = find(US(:,1)>1960);
time = US(indexUS(1:end-1),1); cpiUS = US(indexUS(1:end-1),2); cpiGE = GE(1:end-1,2); cpiJP = JP(1:end,2);
OILUS = cpiOILUS(1:end-1,1); OILGE = cpiOILDM(1:end-1,1); OILJP = cpiOILYEN(1:end-1,1);
DM = DM(1:end-1,1); YEN = YEN(1:end-1,1);

T = length(US);

%Compute annualised inflation rates for horizons of 12, 24, 36 months
for m = 1:12, index = [m:12:length(cpiUS)]; eval(['Y',int2str(m),'US=100*log(cpiUS(index(2:end))./cpiUS(index(1:end-1)));']), end
for m = 1:12, index = [m:12:length(cpiGE)]; eval(['Y',int2str(m),'GE=100*log(cpiGE(index(2:end))./cpiGE(index(1:end-1)));']), end
for m = 1:12, index = [m:12:length(cpiJP)]; eval(['Y',int2str(m),'JP=100*log(cpiJP(index(2:end))./cpiJP(index(1:end-1)));']), end
for m = 1:12, index = [m:12:length(cpiUS)]; eval(['Y',int2str(m),'OILUS=100*log(cpiOILUS(index(2:end))./cpiOILUS(index(1:end-1)));']), end
for m = 1:12, index = [m:12:length(cpiUS)]; eval(['Y',int2str(m),'OILGE=100*log(cpiOILDM(index(2:end))./cpiOILDM(index(1:end-1)));']), end
for m = 1:12, index = [m:12:length(cpiUS)]; eval(['Y',int2str(m),'OILJP=100*log(cpiOILYEN(index(2:end))./cpiOILYEN(index(1:end-1)));']), end
for m = 1:12, index = [m:12:length(cpiUS)]; eval(['Y',int2str(m),'MUS=100*log(MUS(index(2:end))./MUS(index(1:end-1)));']), end
for m = 1:12, index = [m:12:length(cpiUS)]; eval(['Y',int2str(m),'MGE=100*log(MGE(index(2:end))./MGE(index(1:end-1)));']), end
for m = 1:12, index = [m:12:length(cpiUS)]; eval(['Y',int2str(m),'MJP=100*log(MJP(index(2:end))./MJP(index(1:end-1)));']), end
for m = 1:12, index = [m:12:length(cpiUS)]; eval(['Y',int2str(m),'FXdmUS=100*log(DM(index(2:end))./DM(index(1:end-1)));']), end
for m = 1:12, index = [m:12:length(cpiUS)]; eval(['Y',int2str(m),'FXyenUS=100*log(YEN(index(2:end))./YEN(index(1:end-1)));']), end
for m = 1:12, index = [m:12:length(cpiUS)]; eval(['Y',int2str(m),'FXGE=100*log(DM(index(2:end))./DM(index(1:end-1)));']), end
for m = 1:12, index = [m:12:length(cpiUS)]; eval(['Y',int2str(m),'FXJP=100*log(YEN(index(2:end))./YEN(index(1:end-1)));']), end


TABLE1=[]; TABLE2=[]; COUNTRY = strvcat('US','GE','JP'); p=[.25,.75]; 
UT = 3; % Upper threshold for inflation
LT = 1; % Lower threshold for inflation
MT = 2; % Threshold for mean-inflation targeter

% Start the data loop
for c = 3:3 %1:3
    for m = 1:12 %1:12
        eval(['Y=Y', num2str(m), COUNTRY(c,:),';']), eval(['OIL=Y', num2str(m), 'OIL', COUNTRY(c,:),';'])
        eval(['M=Y', num2str(m), 'M', COUNTRY(c,:),';']), 
        if c==1, eval(['FXdm=Y', num2str(m), 'FXdm', COUNTRY(c,:),';']), eval(['FXyen=Y', num2str(m), 'FXyen', COUNTRY(c,:),';'])
        else eval(['FX=Y', num2str(m), 'FX', COUNTRY(c,:),';']), end
        
        if c==1, X=[]; elseif c==2, X=[]; else X=M; end % Choose model as in table 1 of the paper
        
        if ~isempty(X), T = min([length(Y),length(X)]); Y=Y(1:T); X=X(1:T,:); else T=length(Y); Y=Y(1:T); end
        
        % *********************************************************************************************    
        % STEP 1. Model the mean
        if c==1, LAG=3; elseif c==2, LAG=1; else LAG=2; end % Choose model as in table 1 of the paper
        [Yhat, Res, BETA] = LagRegression([Y,X], LAG); 
        Y = Y(LAG+1:T); T=length(Y);
        
        % Ljung-Box statistic.
        [LB(c), LBpvalue(c)] = LjungBox(Res);
        [LB2(c), LBpvalue2(c)] = LjungBox(Res.^2);    
        
        % *********************************************************************************************
        % STEP 2. Compute the risk measures
        if LBpvalue2(c)<0.05
            % Run the garch model.
            garchOut = garchModel(p, Res, Yhat, i, UT, LT);
            param = garchOut.param; 
            if param(2)+param(3)>1 | param(2)<0
                
                for i =1:length(Y)
                    index1 = find(Res<LT-Yhat(i)); index2 = find(Res>UT-Yhat(i));
                    if isempty(index1), PD=0; DR1(i,1)=0; DR2(i,1)=0;
                    else PD=length(index1)/T; 
                        DR1(i,1)=PD*(Yhat(i)-LT+mean(Res(index1)));
                        DR2(i,1)=-PD*((Yhat(i)-LT)^2+mean(Res(index1).^2)+2*(Yhat(i)-LT)*mean(Res(index1)));
                    end
                    
                    if isempty(index2), PEI=0; EIR1(i,1)=0; EIR2(i,1)=0;
                    else PEI=length(index2)/T; 
                        EIR1(i,1)=PEI*(Yhat(i)-UT+mean(Res(index2))); 
                        EIR2(i,1)=PEI*((Yhat(i)-UT)^2+mean(Res(index2).^2)+2*(Yhat(i)-UT)*mean(Res(index2)));
                    end
                end
                
            else DR1 = garchOut.DR1; DR2 = garchOut.DR2; EIR1 = garchOut.EIR1; EIR2 = garchOut.EIR2;
            end
            
        else for i =1:length(Y)
                index1 = find(Res<LT-Yhat(i)); index2 = find(Res>UT-Yhat(i));
                if isempty(index1), PD=0; DR1(i,1)=0; DR2(i,1)=0;
                else PD=length(index1)/T; 
                    DR1(i,1)=PD*(Yhat(i)-LT+mean(Res(index1))); 
                    DR2(i,1)=-PD*((Yhat(i)-LT)^2+mean(Res(index1).^2)+2*(Yhat(i)-LT)*mean(Res(index1)));
                end
                
                if isempty(index2), PEI=0; EIR1(i,1)=0; EIR2(i,1)=0;
                else PEI=length(index2)/T; 
                    EIR1(i,1)=PEI*(Yhat(i)-UT+mean(Res(index2)));
                    EIR2(i,1)=PEI*((Yhat(i)-UT)^2+mean(Res(index2).^2)+2*(Yhat(i)-UT)*mean(Res(index2)));
                end
            end
        end
            
        index=[m:12:12*T]';
        eval(['DR1',COUNTRY(c,:),'(index) = DR1;']), eval(['EIR1',COUNTRY(c,:),'(index) = EIR1;'])
        eval(['DR2',COUNTRY(c,:),'(index) = DR2;']), eval(['EIR2',COUNTRY(c,:),'(index) = EIR2;'])  
        eval(['Y',COUNTRY(c,:),'(index) = Y;']), eval(['Yhat',COUNTRY(c,:),'(index) = Yhat;'])   
        clear DR1 EIR1 DR2 EIR2
    end
end

tt=find(time==1963); TIMEJP = time(tt+1:tt+length(EIR1JP));
for c=3:3
    eval(['i1=find(TIME', COUNTRY(c,:),'>1960 & TIME', COUNTRY(c,:),'<=1970);'])
    eval(['i2=find(TIME', COUNTRY(c,:),'>1970 & TIME', COUNTRY(c,:),'<=1980);'])
    eval(['i3=find(TIME', COUNTRY(c,:),'>1980 & TIME', COUNTRY(c,:),'<=1990);'])
    eval(['i4=find(TIME', COUNTRY(c,:),'>1990 & TIME', COUNTRY(c,:),'<=2000);'])
    eval(['i5=find(TIME', COUNTRY(c,:),'>2000);'])
    for i=1:5
        eval(['TABLE1=[TABLE1,[mean(EIR1',COUNTRY(c,:),'(i',int2str(i),'));mean(DR1',COUNTRY(c,:),'(i',int2str(i),'))]];'])
        eval(['TABLE2=[TABLE2,[mean(EIR2',COUNTRY(c,:),'(i',int2str(i),'));mean(DR2',COUNTRY(c,:),'(i',int2str(i),'))]];'])
    end
end
TABLE1=[reshape(TABLE1(1,:),5,1)'; reshape(TABLE1(2,:),5,1)'];
TABLE2=[reshape(TABLE2(1,:),5,1)'; reshape(TABLE2(2,:),5,1)'];

f = [TIMEJP(end-11:end), YJP(end-11:end)', YhatJP(end-11:end)', 1.5*DR2JP(end-11:end)'+EIR1JP(end-11:end)', DR1JP(end-11:end)'+1.5*EIR2JP(end-11:end)'];

S = find(TIMEJP == 1980); 
TIMEJP = TIMEJP(S:end); YJP = YJP(S:end); YhatJP = YhatJP(S:end); DR1JP = DR1JP(S:end); DR2JP = DR2JP(S:end); EIR1JP = EIR1JP(S:end); EIR2JP = EIR2JP(S:end); 
mX = min(TIMEJP); MX = max(TIMEJP); 
BR11 = DR1JP' + EIR1JP'; BR21 = 1.5*DR2JP' + EIR1JP'; BR12 = DR1JP' + 1.5*EIR2JP';
mY = min(min(DR1JP'+EIR1JP'))-1; MY = max(max(YJP'))+1;
CMDI = min(YhatJP'-LT,0); % Conditional Mean Deflation Indicator
CMII = max(YhatJP'-UT,0); % Conditional Mean Inflation Indicator

figure, plot(TIMEJP, YJP, '-', TIMEJP, CMDI+CMII, ':', TIMEJP, BR11, '-.', TIMEJP, zeros(length(TIMEJP),1))
    axis manual, axis([mX, MX, mY, MY]), legend('Actual Inflation', 'Conditional Mean Measure', 'BR_1_,_1')

mY = min(min(BR21))-1; MY = max(BR12)+1;
figure, plot(TIMEJP, YJP, '-', TIMEJP, BR11, ':', TIMEJP, BR21, '-.', TIMEJP, BR12, '--', TIMEJP, zeros(length(TIMEJP),1))
    axis manual, axis([mX, MX, mY, MY]), legend('Actual Inflation', 'BR_1_,_1', 'BR_2_,_1', 'BR_1_,_2')
