function garchOut = garchModel(y, yhat)
%
% Estimate a GARCH(1,1) model (with robust standard errors and LB statistics) and computes the various risk indices.
% INPUT: y    = vector of residuals from the regression model
%        yhat = OLS regression estimates 

T = length(y);

% Run the garch model.
f = garch(y);

param = f.theta;
stdErr = sqrt(f.variance); variance = f.variance;
VAR4cast = [1, variance(T), y(T)^2] * param;
rse = garch11robustStdErr(param, y); % Robust standard errors
stdRes = y./stdErr;
[LBstd, LBpvaluestd] = LjungBox(stdRes); % Ljung-Box statistic on residuals
[LBstd2, LBpvaluestd2] = LjungBox(stdRes.^2); % Ljung-Box statistic on squared residuals


garchOut.param = param;
garchOut.LBstd = LBstd;
garchOut.LBpvaluestd = LBpvaluestd;
garchOut.LBstd2 = LBstd2;
garchOut.LBpvaluestd2 = LBpvaluestd2;
garchOut.variance = variance;
